package info.textgrid.lab.collatex.process;

import info.textgrid.lab.collatex.model.equivalences.EToken;
import info.textgrid.lab.collatex.model.equivalences.Equivalence;
import info.textgrid.lab.collatex.model.equivalences.Equivalences;

import java.util.Map;

import com.google.common.collect.Maps;

import eu.interedition.collatex.Token;
import eu.interedition.collatex.matching.EqualityTokenComparator;
import eu.interedition.collatex.simple.SimpleToken;

/**
 * An comparator based on a {@link Equivalences} normalization. This
 * implementation is semantically equivalent to the one specified in the package
 * description of {@link info.textgrid.lab.collatex.model.equivalences}.
 */
public class EquivalenceSetTokenComparator extends EqualityTokenComparator {

	private final Equivalences equivalences;
	private final Map<String, Equivalence> tokenIndex;

	public EquivalenceSetTokenComparator(final Equivalences equivalences) {
		super();
		this.equivalences = equivalences;

		tokenIndex = Maps.newHashMap();
		for (final Equivalence equivalence : equivalences.getEquivalences())
			for (final EToken token : equivalence.getTokens())
				tokenIndex.put(token.getNormalizedContent(), equivalence);
	}

	public Equivalences getEquivalences() {
		return equivalences;
	}

	@Override
	public int compare(final Token base, final Token witness) {
		final int result = super.compare(base, witness);
		if (result != 0) {
			final String normalized = ((SimpleToken) base).getNormalized();
			if (tokenIndex.containsKey(normalized)) {
				final Equivalence equivalence = tokenIndex.get(normalized);
				if (equivalence.isLocal()) {
					final EToken baseToken = new EToken(base);
					if (equivalence.getTokens().contains(baseToken)) {
						final EToken witnessToken = new EToken(witness);
						if (equivalence.getTokens().contains(witnessToken))
							//							System.out.println(base + " equals " + witness
							//									+ " due to " + equivalence);
							return 0;
					}
				} else
					for (final EToken token : equivalence.getTokens())
						if (token.getNormalizedContent().equals(
								((SimpleToken) witness).getNormalized()))
							//							System.out.println(base + " equals " + witness
							//									+ " due to " + equivalence);
							return 0;
			}
		}
		return result;
	}

}
